/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/
#include "prefs.h"
//#include "settings.h"
#include "kalvanewstuff.h"

#include <qstring.h>
#include <qregexp.h>
#include <qapplication.h>

#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kconfigbase.h>
#include <kstandarddirs.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <klistbox.h>
#include <kprocess.h>
#include <kdirlister.h>
#include <kmessagebox.h>
#include <kdirwatch.h>

#include <qdir.h>
#include <cmath>

Prefs::Prefs()
	: Prefs_base()
{
   m_newStuff = 0;
   m_proc = 0;
//   m_chlsts_dir = locateLocal ("data", "kalva/chlst/", true ); 
   m_chlsts_dir = KGlobal::dirs()->saveLocation("data", "kalva/chlst", true);
   kdDebug() << "chlstdir = " << m_chlsts_dir << endl;
   kdw   = new KDirWatch( this );
   kdw->addDir( m_chlsts_dir, true );
   loadAudiodevList();
   loadChlstChooser();

   //monitor destDir for changes; reload chlstChooser when files are created.
   connect( kdw,  SIGNAL( dirty( const QString & ) ),
            this, SLOT(   slotLoadChlstChooser( const QString & ) ) );
}

void Prefs::setAudiodevList(KProcess *, char* stdOutAudiodevs, int)
{

   QString OutAudiodevs = (QString)stdOutAudiodevs;
   OutAudiodevs.remove((QRegExp)"\\s\\d+");
   OutAudiodevs.remove((QRegExp)",");
   m_audiodevs = QStringList::split((QRegExp)"[PR]?\n",
                                  OutAudiodevs);
   m_audiodev->insertStringList(m_audiodevs,-1);
   m_audiodev->setCurrentItem(kcfg_audiodev->text());
}

void Prefs::loadAudiodevList()
{
   m_audiodev->clear();
   m_audiodev->insertItem(i18n("[don't use aumix]"),-1);
   if (!m_proc)
   	m_proc = new KProcess;
   (*m_proc) << "aumix" << "-q";
   QApplication::connect(m_proc, SIGNAL(processExited(KProcess *)),
                         this, SLOT(aumixExited(KProcess *)));
   QApplication::connect(m_proc, SIGNAL(receivedStdout(KProcess *, char*, int)),
                         this, SLOT(setAudiodevList(KProcess *, char*, int)));
   QApplication::connect(m_proc, SIGNAL(receivedStderr(KProcess *, char*, int)),
                         this, SLOT(setAudiodevList(KProcess *, char*, int)));
   m_proc->start(KProcess::NotifyOnExit, KProcess::Stdout);
}

void Prefs::aumixExited(KProcess *proc)
{ 
   if (proc->normalExit() == true) {
      kdDebug() << "Audiodevs loaded" << endl;
   }  else  {
      kdWarning() << "aumix exited with status" << proc->exitStatus() << endl;
      kdWarning() << "aumix exited abnormal" << endl;
   }
}

void Prefs::slotLoadChlstChooser( const QString &path )
{
   kdDebug() << i18n("Content of %1 has changed").arg(path) << endl;
   this->loadChlstChooser();
}

void Prefs::loadChlstChooser()
{
   ///@TODO read comment from the active file
   kdDebug() << i18n(QString("Going to read channellists directory %1.").arg(m_chlsts_dir)) << endl;

   m_chlstChooser->clear();
   QDir d = QDir(m_chlsts_dir);
   if ( !d.exists() )
   {
       kdWarning() << i18n(QString("Cannot find the channellists directory %1!").arg(m_chlsts_dir)) << endl;
       return;
   }
   kdDebug() << i18n(QString("Read channellists directory %1.").arg(m_chlsts_dir)) << endl;
   d.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
   QStringList chlsts = d.entryList();
   m_chlstChooser->insertStringList(chlsts,-1);

   //QString chlst = Settings::chlst();
   QString chlst = kcfg_chlst->text();
   kdDebug() << i18n(QString("Read in chlst %1.").arg(chlst)) << endl;
   QListBoxItem* i = m_chlstChooser->findItem(chlst);
   m_chlstChooser->setCurrentItem(i);
   kdDebug() << "Channelllists loaded" << endl;
}

void Prefs::setChannellist(const QString & chlst)
{
   kcfg_chlst->setText(chlst);
   kcfg_channellist->setText(QString("%1/%2").arg(m_chlsts_dir).arg(chlst));
   m_chlstLocation->clear();
   m_chlstLocation->insert(chlst.section(".",0,0));
   m_chlstTvType->setCurrentItem(chlst.section(".",1,1));
}

void Prefs::slotDeleteChlst()
{
    QString filename(kcfg_channellist->text());
    QFile file(filename);
    QString msg = i18n("This can not be undone.\n"
                       "Do You realy want to delete this channellist?" );
    switch( KMessageBox::warningYesNo( this, msg, QString::null,
                           KStdGuiItem::del(), KStdGuiItem::discard() ) )
    {
      case KMessageBox::Yes:
      {
         kdDebug() << "deleting " << filename << endl;
         file.remove();
         KConfigGroup knsConfig(KGlobal::config(), "KNewStuffStatus");
	 knsConfig.deleteEntry( kcfg_chlst->text() );
         KGlobal::config()->sync();
	 return;
      }
      case KMessageBox::No: // Don't save and don't close.
	return;
    }
}

void Prefs::slotDownloadChlst()
{
   if ( !m_newStuff )
        m_newStuff = new KalvaNewStuff( this );
    m_newStuff->download();
}

void Prefs::slotUploadChlst()
{
//    if ( !m_newStuff )
//         m_newStuff = new KalvaNewStuff( this );
//     QString chlst = kcfg_channellist->text();
//     m_newStuff->upload( chlst, chlst );
   kdDebug() << i18n( "Uploading data is not possible yet!" ) << endl;
}

void Prefs::slotSetCrop( const QString & res )
{
   kdDebug() << i18n( "computing crop values" ) << endl;
   QString   resw   = res.section( ":", 0, 0 );
   QString   resh   = res.section( ":", 1, 1 );
   resh             = resh.section(" ", 0, 0 );
   kdDebug() << "resw  = " << resw << endl;
   kdDebug() << "resh  = " << resh << endl;
   int       realw  = resw.toInt();
   int       realh  = resh.toInt();
   kdDebug() << "realw  = " << realw << endl;
   kdDebug() << "realh  = " << realh << endl;
   float overscan   = 0.035;
   float exactw     = realw - (2 * realw * overscan);
   float exacth     = realh - (2 * realh * overscan);
   kdDebug() << "exactw  = " << exactw << endl;
   kdDebug() << "exacth  = " << exacth << endl;

   double modw16    = std::fmod(exactw, 16);
   kdDebug() << "modw16  = " << modw16 << endl;
   double cropw     = exactw - modw16;
   if ( modw16 > 8 )
      cropw += 16;
   kdDebug() << "cropw  = " << cropw << endl;
   double modh16    = std::fmod(exacth, 16);
   kdDebug() << "modh16  = " << modh16 << endl;
   double croph     = exacth - modh16;
   if ( modh16 > 8 )
      croph += 16;
   kdDebug() << "croph  = " << croph << endl;
   double overscanw = (realw - cropw) / 2;
   double overscanh = (realh - croph) / 2;
   kdDebug() << "overscanw  = " << overscanw << endl;
   kdDebug() << "overscanh  = " << overscanh << endl;

   QString crop   = QString("%1:%2:%3:%4")
                    .arg(cropw).arg(croph)
                    .arg(overscanw).arg(overscanh);
//   double aspect = exactw / exacth;
   double aspect_capture = exactw / exacth;
   kdDebug() << "aspect_capture  = " << aspect_capture << endl;
   double aspect_pal = 1.333333;
   kdDebug() << "aspect_pal  = " << aspect_pal<< endl;
   double aspect_crop = cropw / croph;
   kdDebug() << "aspect_crop  = " << aspect_crop<< endl;
   double aspect = aspect_pal * aspect_crop / aspect_capture;
   kdDebug() << "aspect  = " << aspect<< endl;
   kcfg_crop->setText(crop);
   kcfg_aspect->setText(QString("%1").arg(aspect));
   return;
}

